@@ -10,7 +10,8 @@ from openmdao.core.constants import INT_DTYPE
 from openmdao.core.explicitcomponent import ExplicitComponent
 from openmdao.utils.units import valid_units
 from openmdao.utils import cs_safe
-from openmdao.utils.om_warnings import issue_warning, DerivativesWarning, warn_deprecation
+from openmdao.utils.om_warnings import issue_warning, DerivativesWarning, warn_deprecation, \
+    SetupWarning
 
 # regex to check for variable names.
 VAR_RGX = re.compile(r'([.]*[_a-zA-Z]\w*[ ]*\(?)')
@@ -18,10 +19,11 @@ VAR_RGX = re.compile(r'([.]*[_a-zA-Z]\w*[ ]*\(?)')
 # Names of metadata entries allowed for ExecComp variables.
 _allowed_meta = {'value', 'val', 'shape', 'units', 'res_units', 'desc',
                  'ref', 'ref0', 'res_ref', 'lower', 'upper', 'src_indices',
-                 'flat_src_indices', 'tags', 'shape_by_conn', 'copy_shape'}
+                 'flat_src_indices', 'tags', 'shape_by_conn', 'copy_shape', 'constant'}
 
 # Names that are not allowed for input or output variables (keywords for options)
-_disallowed_names = {'has_diag_partials', 'units', 'shape', 'shape_by_conn', 'run_root_only'}
+_disallowed_names = {'has_diag_partials', 'units', 'shape', 'shape_by_conn', 'run_root_only',
+                     'constant'}
 
 
 def check_option(option, value):
@@ -102,6 +104,9 @@ class ExecComp(ExplicitComponent):
     _requires_fd : dict
         Contains a mapping of 'of' variables to a tuple of the form (wrts, functs) for those
         'of' variables that require finite difference to be used to compute their derivatives.
+    _constants : dict of dicts
+        Constants defined in the expressions. The key is the name of the constant and the value
+        is a dict of metadata.
     """
 
     def __init__(self, exprs=[], **kwargs):
@@ -219,6 +224,8 @@ class ExecComp(ExplicitComponent):
         self._manual_decl_partials = False
         self._no_check_partials = True
 
+        self._constants = {}
+
     def initialize(self):
         """
         Declare options.
@@ -308,7 +315,7 @@ class ExecComp(ExplicitComponent):
         allvars = set()
 
         self._exprs_info = exprs_info = [(self._parse_for_out_vars(expr.split('=', 1)[0]),
-                                          self._parse_for_names(expr)) for expr in exprs]
+                                          self._parse_for_names(expr, **kwargs)) for expr in exprs]
 
         self._requires_fd = {}
 
@@ -334,6 +341,20 @@ class ExecComp(ExplicitComponent):
 
         # make sure all kwargs are legit
         for arg, val in kwargs.items():
+
+            if isinstance(val, dict) and 'constant' in val and val['constant']:
+                if 'val' not in val:
+                    raise RuntimeError(f"{self.msginfo}: arg '{arg}' in call to ExecComp() "
+                                       "is a constant but no value is given")
+                for ignored_meta in ['units', 'shape']:
+                    if ignored_meta in val:
+                        issue_warning(f"arg '{arg}' in call to ExecComp() "
+                                      f"is a constant. The {ignored_meta} will be ignored",
+                                      prefix=self.msginfo, category=SetupWarning)
+
+                self._constants[arg] = val['val']
+                continue  # TODO should still do some checking here!
+
             if arg not in allvars:
                 raise RuntimeError("%s: arg '%s' in call to ExecComp() "
                                    "does not refer to any variable in the "
@@ -441,6 +462,8 @@ class ExecComp(ExplicitComponent):
                 if var in outs:
                     current_meta = self.add_output(var, val, **meta)
                 else:
+                    if 'constant' in meta:
+                        meta.pop('constant', None)
                     current_meta = self.add_input(var, val, **meta)
 
             if var not in init_vals:
@@ -512,9 +535,16 @@ class ExecComp(ExplicitComponent):
                                 "function or constant." % (self.msginfo, v))
         return vnames
 
-    def _parse_for_names(self, s):
+    def _parse_for_names(self, s, **kwargs):
         names = [x.strip() for x in re.findall(VAR_RGX, s) if not x.startswith('.')]
-        vnames = set([n for n in names if not n.endswith('(')])
+        vnames = set()
+        for n in names:
+            if n.endswith('('):
+                continue
+            if n in kwargs and isinstance(kwargs[n], dict) and 'constant' in kwargs[n] \
+                    and kwargs[n]['constant']:
+                continue
+            vnames.add(n)
         fnames = [n[:-1] for n in names if n[-1] == '(']
         to_remove = []
         for v in vnames:
@@ -652,7 +682,9 @@ class ExecComp(ExplicitComponent):
         """
         for i, expr in enumerate(self._codes):
             try:
-                exec(expr, _expr_dict, _IODict(outputs, inputs))  # nosec: limited to _expr_dict
+                #  inputs, outputs, and _constants are vectors
+                exec(expr, _expr_dict, _IODict(outputs, inputs, self._constants))  # nosec:
+                # limited to _expr_dict
             except Exception as err:
                 raise RuntimeError(f"{self.msginfo}: Error occurred evaluating '{self._exprs[i]}':"
                                    f"\n{err}")
@@ -818,9 +850,11 @@ class _IODict(object):
         The inputs object to be wrapped.
     _outputs : dict-like
         The outputs object to be wrapped.
+    _constants : dict-like
+        The constants object to be wrapped.
     """
 
-    def __init__(self, outputs, inputs):
+    def __init__(self, outputs, inputs, constants):
         """
         Create the dict wrapper.
 
@@ -831,21 +865,28 @@ class _IODict(object):
 
         inputs : dict-like
             The inputs object to be wrapped.
+
+        constants : dict-like
+            The constants object to be wrapped.
         """
         self._outputs = outputs
         self._inputs = inputs
+        self._constants = constants
 
     def __getitem__(self, name):
         try:
             return self._inputs[name]
         except KeyError:
-            return self._outputs[name]
+            try:
+                return self._outputs[name]
+            except KeyError:
+                return self._constants[name]
 
     def __setitem__(self, name, value):
         self._outputs[name] = value
 
     def __contains__(self, name):
-        return name in self._outputs or name in self._inputs
+        return name in self._outputs or name in self._inputs or name in self._constants
 
 
 def _import_functs(mod, dct, names=None):
@@ -901,8 +942,8 @@ except ImportError:
 else:
     _import_functs(scipy.special, _expr_dict, names=['erf', 'erfc'])
 
-    from distutils.version import LooseVersion
-    if LooseVersion(scipy.__version__) >= LooseVersion("1.5.0"):
+    from packaging.version import Version
+    if Version(scipy.__version__) >= Version("1.5.0"):
         def factorial(*args):
             """
             Raise a RuntimeError stating that the factorial function is not supported.
