@@ -9,8 +9,10 @@ import numpy as np
 from openmdao.components.interp_util.interp_akima import InterpAkima, Interp1DAkima
 from openmdao.components.interp_util.interp_bsplines import InterpBSplines
 from openmdao.components.interp_util.interp_cubic import InterpCubic
-from openmdao.components.interp_util.interp_lagrange2 import InterpLagrange2, Interp3DLagrange2
-from openmdao.components.interp_util.interp_lagrange3 import InterpLagrange3, Interp3DLagrange3
+from openmdao.components.interp_util.interp_lagrange2 import InterpLagrange2, Interp3DLagrange2,\
+    Interp2DLagrange2, Interp1DLagrange2
+from openmdao.components.interp_util.interp_lagrange3 import InterpLagrange3, Interp3DLagrange3, \
+    Interp2DLagrange3, Interp1DLagrange3
 from openmdao.components.interp_util.interp_scipy import InterpScipy
 from openmdao.components.interp_util.interp_slinear import InterpLinear, Interp3DSlinear, \
     Interp1DSlinear, Interp2DSlinear
@@ -33,7 +35,11 @@ INTERP_METHODS = {
     '1D-slinear': Interp1DSlinear,
     '2D-slinear': Interp2DSlinear,
     '3D-slinear': Interp3DSlinear,
+    '1D-lagrange2': Interp1DLagrange2,
+    '2D-lagrange2': Interp2DLagrange2,
     '3D-lagrange2': Interp3DLagrange2,
+    '1D-lagrange3': Interp1DLagrange3,
+    '2D-lagrange3': Interp2DLagrange3,
     '3D-lagrange3': Interp3DLagrange3,
     '1D-akima': Interp1DAkima,
     'trilinear': Interp3DSlinear,  # Deprecated
@@ -45,7 +51,8 @@ TABLE_METHODS = ['slinear', 'lagrange2', 'lagrange3', 'cubic', 'akima',
                  'trilinear', 'akima1D',  # These two are Deprecated
                  '3D-slinear', '2D-slinear', '1D-slinear',
                  '1D-akima',
-                 '3D-lagrange2', '3D-lagrange3']
+                 '3D-lagrange2', '2D-lagrange2', '1D-lagrange2',
+                 '3D-lagrange3', '2D-lagrange3', '1D-lagrange3']
 SPLINE_METHODS = ['slinear', 'lagrange2', 'lagrange3', 'cubic', 'akima', 'bsplines',
                   'scipy_cubic', 'scipy_slinear', 'scipy_quintic']
 
