@@ -163,9 +163,13 @@ def run_opt(driver_class, mode, assemble_type=None, color_info=None, derivs=True
     if 'method' in options:
         p.model.approx_totals(method=options['method'])
         del options['method']
+        
+    min_improve_pct = options.get('min_improve_pct', 5.)
+    if 'min_improve_pct' in options:
+        del options['min_improve_pct']
 
     if 'dynamic_total_coloring' in options:
-        p.driver.declare_coloring(tol=1e-15)
+        p.driver.declare_coloring(tol=1e-15, min_improve_pct=min_improve_pct)
         del options['dynamic_total_coloring']
 
     p.driver.options.update(options)
@@ -277,13 +281,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
     def test_dynamic_total_coloring_snopt_auto_autoivc(self):
@@ -296,13 +302,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
     def test_dynamic_total_coloring_snopt_auto_dyn_partials(self):
@@ -314,13 +322,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
 
         partial_coloring = p_color.model._get_subsystem('arctan_yox')._coloring_info['coloring']
         expected = [
@@ -345,16 +355,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
 
-        # This has been changed to greater or equal to 28 iterations. This change arose when updating
-        # to pyOptSparse v2.1.0 and SNOPT 7.7
-        self.assertGreaterEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
     def test_dynamic_total_coloring_snopt_auto_assembled(self):
@@ -366,13 +375,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
     def test_dynamic_fwd_simul_coloring_snopt_approx_cs(self):
@@ -385,12 +396,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - fwd coloring saves 16 nonlinear solves per driver iter  (6 vs 22).
-        # - dynamic coloring takes 66 nonlinear solves (22 each for 3 full jacs)
-        # - (total_solves - 2) / (solves_per_iter) should be equal to
-        #       (total_color_solves - 2 - dyn_solves) / color_solves_per_iter
-        self.assertEqual((p.model._solve_nl_count - 2) / 22,
-                         (p_color.model._solve_nl_count - 2 - 66) / 6)
+        p.model._solve_nl_count = 0
+        p_color.model._solve_nl_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_nl_count, 21)
+        self.assertEqual(p_color.model._solve_nl_count, 5)
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
     def test_dynamic_fwd_simul_coloring_snopt_approx_fd(self):
@@ -403,13 +417,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
+        p.model._solve_nl_count = 0
+        p_color.model._solve_nl_count = 0
 
-        # - fwd coloring saves 16 nonlinear solves per driver iter  (6 vs 22).
-        # - dynamic coloring takes 66 nonlinear solves (22 each for 3 full jacs)
-        # - (total_solves - 2) / (solves_per_iter) should be equal to
-        #       (total_color_solves - 2 - dyn_solves) / color_solves_per_iter
-        self.assertEqual((p.model._solve_nl_count - 2) / 22,
-                         (p_color.model._solve_nl_count - 2 - 66) / 6)
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_nl_count, 21)
+        self.assertEqual(p_color.model._solve_nl_count, 5)
 
     def test_size_zero_array_in_component(self):
         class DynamicPartialsComp(om.ExplicitComponent):
@@ -512,13 +528,15 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         p = run_opt(pyOptSparseDriver, 'auto', optimizer='SLSQP', print_results=False)
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
 
         # test __repr__
         rep = repr(p_color.driver._coloring_info['coloring'])
@@ -591,17 +609,20 @@ class SimulColoringPyoptSparseRevTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - rev coloring saves 11 solves per driver iter  (11 vs 22)
-        # - initial solve for linear constraints takes 1 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 22 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 1 for the uncolored case and 22 * 3 + 1 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 1) / 22,
-                         (p_color.model._solve_count - 1 - 22 * 3) / 11)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 11 solves per driver iter  (11 vs 22)
+        self.assertEqual(p.model._solve_count, 22)
+        self.assertEqual(p_color.model._solve_count, 11)
 
         # improve coverage of coloring.py
         coloring = p_color.driver._coloring_info['coloring']
         coloring.display_txt()
+        coloring.display(show=False)
         with open(os.devnull, 'w') as f:
             array_viz(coloring.get_dense_sparsity(), prob=p_color, stream=f)
             array_viz(coloring.get_dense_sparsity(), stream=f)
@@ -628,13 +649,15 @@ class SimulColoringPyoptSparseRevTestCase(unittest.TestCase):
         p = run_opt(pyOptSparseDriver, 'rev', optimizer='SLSQP', print_results=False)
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 11 solves per driver iter  (11 vs 22)
-        # - initial solve for linear constraints takes 1 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 22 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 1 for the uncolored case and 22 * 3 + 1 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 1) / 22,
-                         (p_color.model._solve_count - 1 - 22 * 3) / 11)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 11 solves per driver iter  (11 vs 22)
+        self.assertEqual(p.model._solve_count, 22)
+        self.assertEqual(p_color.model._solve_count, 11)
 
 
 @use_tempdirs
@@ -662,13 +685,33 @@ class SimulColoringScipyTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - bidirectional coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
+        
+    def test_min_improve_pct(self):
+        # first, run w/o coloring
+        p = run_opt(om.ScipyOptimizeDriver, 'auto', optimizer='SLSQP', disp=False)
+        p_color = run_opt(om.ScipyOptimizeDriver, 'auto', optimizer='SLSQP', disp=False, dynamic_total_coloring=True, min_improve_pct=99.)
+
+        assert_almost_equal(p['circle.area'], np.pi, decimal=7)
+        assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
+
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring should have been aborted since it couldn't improve more than 99% so solve counts should match
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 21)
 
     def test_dynamic_total_coloring_auto_con_alias(self):
         # This test makes sure that coloring works with aliased constraints.
@@ -680,13 +723,15 @@ class SimulColoringScipyTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - bidirectional coloring saves 16 solves per driver iter  (5 vs 21)
-        # - initial solve for linear constraints takes 21 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 21) / 21,
-                         (p_color.model._solve_count - 21 * 4) / 5)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (5 vs 21)
+        self.assertEqual(p.model._solve_count, 21)
+        self.assertEqual(p_color.model._solve_count, 5)
 
     def test_problem_total_coloring_auto(self):
 
@@ -963,13 +1008,15 @@ class SimulColoringRevScipyTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - rev coloring saves 11 solves per driver iter  (11 vs 22)
-        # - initial solve for linear constraints takes 1 in both cases (only done once)
-        # - dynamic case does 3 full compute_totals to compute coloring, which adds 22 * 3 solves
-        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
-        # - where N is 1 for the uncolored case and 22 * 3 + 1 for the dynamic colored case.
-        self.assertEqual((p.model._solve_count - 1) / 22,
-                         (p_color.model._solve_count - 1 - 22 * 3) / 11)
+        p.model._solve_count = 0
+        p_color.model._solve_count = 0
+
+        J = p.driver._compute_totals()
+        J_color = p_color.driver._compute_totals()
+
+        # coloring saves 16 solves per driver iter  (11 vs 22)
+        self.assertEqual(p.model._solve_count, 22)
+        self.assertEqual(p_color.model._solve_count, 11)
 
     def test_dynamic_total_coloring_no_derivs(self):
         with self.assertRaises(Exception) as context:
