@@ -8,6 +8,7 @@ from io import StringIO
 import numpy as np
 
 import openmdao.api as om
+from openmdao.core.problem import _default_prob_name
 from openmdao.core.driver import Driver
 from openmdao.test_suite.components.paraboloid import Paraboloid
 from openmdao.test_suite.components.sellar import SellarDerivatives, SellarDerivativesConnected
@@ -522,6 +523,7 @@ class TestProblem(unittest.TestCase):
         prob = om.Problem()
         prob.model = SellarDerivatives()
         prob.model.nonlinear_solver = om.NonlinearBlockGS()
+        prob.model.linear_solver = om.ScipyKrylov()
 
         prob.setup(mode=mode)
         prob.run_model()
@@ -1959,6 +1961,26 @@ class TestProblem(unittest.TestCase):
         self.assertRegex(output[12], r'^\s+upper:')
         self.assertRegex(output[13], r'^\s+array+\(+\[[0-9., e+-]+\]+\)')
 
+    def test_list_problem_vars_before_final_setup(self):
+        prob = om.Problem()
+        prob.model.add_subsystem('parab', Paraboloid(), promotes_inputs=['x', 'y'])
+        prob.model.add_subsystem('const', om.ExecComp('g = x + y'), promotes_inputs=['x', 'y'])
+        prob.model.set_input_defaults('x', 3.0)
+        prob.model.set_input_defaults('y', -4.0)
+        prob.driver = om.ScipyOptimizeDriver()
+        prob.driver.options['optimizer'] = 'COBYLA'
+        prob.model.add_design_var('x', lower=-50, upper=50)
+        prob.model.add_design_var('y', lower=-50, upper=50)
+        prob.model.add_objective('parab.f_xy')
+        prob.model.add_constraint('const.g', lower=0, upper=10.)
+        prob.setup()
+
+        msg = "Problem .*: Problem.list_problem_vars\(\) cannot be called before " \
+                         "`Problem\.run_model\(\)`, `Problem\.run_driver\(\)`, or " \
+                         "`Problem\.final_setup\(\)`\."
+        with self.assertRaisesRegex(RuntimeError, msg):
+            prob.list_problem_vars()
+
     def test_list_problem_w_multi_constraints(self):
         p = om.Problem()
 
@@ -2312,7 +2334,7 @@ class NestedProblemTestCase(unittest.TestCase):
 
             def solve(self):
                 # create a simple subproblem and run it to test for global solver_info bug
-                p = om.Problem(name=self.prob_name)
+                p = om.Problem(name=self.prob_name, reports=False)
                 self._problem = p
                 p.model.add_subsystem('indep', om.IndepVarComp('x', 1.0))
                 p.model.add_subsystem('comp', om.ExecComp('y=2*x'))
@@ -2325,8 +2347,8 @@ class NestedProblemTestCase(unittest.TestCase):
                 return super().solve()
 
         # Initially use the default names
-        openmdao.core.problem._problem_names = []  # need to reset these to simulate separate runs
-        p = om.Problem()
+        openmdao.core.problem._clear_problem_names()  # need to reset these to simulate separate runs
+        p = om.Problem(reports=False)
         p.model.add_subsystem('indep', om.IndepVarComp('x', 1.0))
         G = p.model.add_subsystem('G', om.Group())
         G.add_subsystem('comp', om.ExecComp('y=2*x'))
@@ -2335,26 +2357,28 @@ class NestedProblemTestCase(unittest.TestCase):
         p.setup()
         p.run_model()  # need to do run_model in this test so sub problem is created
 
-        self.assertEqual(p._get_inst_id(), 'problem1')
-        self.assertEqual(G.nonlinear_solver._problem._get_inst_id(), 'problem2')
+        defname = _default_prob_name()
+
+        self.assertEqual(p._get_inst_id(), defname)
+        self.assertEqual(G.nonlinear_solver._problem._get_inst_id(), defname + '2')
 
         # If the second Problem uses the default name of the first
-        openmdao.core.problem._problem_names = []  # need to reset these to simulate separate runs
-        p = om.Problem()
+        openmdao.core.problem._clear_problem_names()  # need to reset these to simulate separate runs
+        p = om.Problem(reports=False)
         p.model.add_subsystem('indep', om.IndepVarComp('x', 1.0))
         G = p.model.add_subsystem('G', om.Group())
         G.add_subsystem('comp', om.ExecComp('y=2*x'))
-        G.nonlinear_solver = _ProblemSolver(prob_name='problem1')
+        G.nonlinear_solver = _ProblemSolver(prob_name=defname)
         p.model.connect('indep.x', 'G.comp.x')
         p.setup()
 
         with self.assertRaises(Exception) as context:
             p.run_model()
-        self.assertEqual(str(context.exception), "The problem name 'problem1' already exists")
+        self.assertEqual(str(context.exception), f"The problem name '{defname}' already exists")
 
         # If the first Problem uses the default name of 'problem2'
-        openmdao.core.problem._problem_names = []  # need to reset these to simulate separate runs
-        p = om.Problem(name='problem2')
+        openmdao.core.problem._clear_problem_names()  # need to reset these to simulate separate runs
+        p = om.Problem(name=defname + '2', reports=False)
         p.model.add_subsystem('indep', om.IndepVarComp('x', 1.0))
         G = p.model.add_subsystem('G', om.Group())
         G.add_subsystem('comp', om.ExecComp('y=2*x'))
@@ -2363,13 +2387,13 @@ class NestedProblemTestCase(unittest.TestCase):
         p.setup()
         p.run_model()
 
-        self.assertEqual(p._get_inst_id(), 'problem2')
-        self.assertEqual(G.nonlinear_solver._problem._get_inst_id(), 'problem2.1')
+        self.assertEqual(p._get_inst_id(), defname + '2')
+        self.assertEqual(G.nonlinear_solver._problem._get_inst_id(),  defname + '2.1')
 
 
 class SystemInTwoProblemsTestCase(unittest.TestCase):
     def test_2problems(self):
-        prob = om.Problem()
+        prob = om.Problem(reports=False)
         G1 = prob.model.add_subsystem("G1", om.Group())
         G2 = G1.add_subsystem('G2', om.Group(), promotes_inputs=['x'])
         G2.add_subsystem('C1', om.ExecComp('y = 2 * x'), promotes_inputs=['x'])
@@ -2379,7 +2403,7 @@ class SystemInTwoProblemsTestCase(unittest.TestCase):
         prob.run_model()
 
         # 2nd problem
-        prob = om.Problem()
+        prob = om.Problem(reports=False)
         prob.model = G2
 
         prob.setup()
