@@ -300,18 +300,6 @@ class TestSystem(unittest.TestCase):
         with assert_warning(OMDeprecationWarning, msg):
             self.assertEqual(meta['comp.a']['value'], 1)
 
-    def test_recording_options_deprecated(self):
-        prob = Problem()
-        msg = "The recording option, record_model_metadata, on System is deprecated. " \
-              "Recording of model metadata will always be done"
-        with assert_warning(OMDeprecationWarning, msg):
-            prob.model.recording_options['record_model_metadata'] = True
-
-        msg = "The recording option, record_metadata, on System is deprecated. " \
-              "Recording of metadata will always be done"
-        with assert_warning(OMDeprecationWarning, msg):
-            prob.model.recording_options['record_metadata'] = True
-
     def test_setup_check_group(self):
 
         class CustomGroup(Group):
@@ -361,6 +349,43 @@ class TestSystem(unittest.TestCase):
 
         self.assertEqual(str(cm.exception), "'comp' <class BadComp>: You forget to call super() in setup()")
 
+    def test_missing_source(self):
+        prob = Problem()
+        root = prob.model
+
+        root.add_subsystem('initial_comp', ExecComp(['x = 10']), promotes_outputs=['x'])
+
+        prob.setup()
+
+        with self.assertRaises(KeyError) as cm:
+            root.get_source('f')
+
+        self.assertEqual(cm.exception.args[0], "<model> <class Group>: source for 'f' not found.")
+
+    def test_list_inputs_before_final_setup(self):
+        class SpeedComp(ExplicitComponent):
+
+            def setup(self):
+                self.add_input('distance', val=1.0, units='km')
+                self.add_input('time', val=1.0, units='h')
+                self.add_output('speed', val=1.0, units='km/h')
+
+            def compute(self, inputs, outputs):
+                outputs['speed'] = inputs['distance'] / inputs['time']
+
+        prob = Problem()
+        prob.model.add_subsystem('c1', SpeedComp(), promotes=['*'])
+        prob.model.add_subsystem('c2', ExecComp('f=speed',speed={'units': 'm/s'}), promotes=['*'])
+
+        prob.setup()
+
+        msg = ("Calling `list_inputs` before `final_setup` will only "
+              "display the default values of variables and will not show the result of "
+              "any `set_val` calls.")
+
+        with assert_warning(UserWarning, msg):
+            prob.model.list_inputs(units=True, prom_name=True)
+
 
 if __name__ == "__main__":
     unittest.main()
