@@ -35,6 +35,10 @@ class DOEDriver(Driver):
         The MPI communicator for the Problem.
     _color : int or None
         In MPI, the cached color is used to determine which cases to run on this proc.
+    _indep_list : list
+        List of design variables.
+    _quantities : list
+        Contains the objectives plus nonlinear constraints.
     """
 
     def __init__(self, generator=None, **kwargs):
@@ -62,6 +66,7 @@ class DOEDriver(Driver):
 
         # What we don't support
         self.supports['distributed_design_vars'] = False
+        self.supports['optimization'] = False
         self.supports._read_only = True
 
         if generator is not None:
@@ -71,6 +76,10 @@ class DOEDriver(Driver):
         self._problem_comm = None
         self._color = None
 
+        self._indep_list = []
+        self._quantities = []
+        self._total_jac_format = 'dict'
+
     def _declare_options(self):
         """
         Declare options before kwargs are processed in the init method.
@@ -155,10 +164,25 @@ class DOEDriver(Driver):
             Failure flag; True if failed to converge, False is successful.
         """
         self.iter_count = 0
+        self._quantities = []
 
         # set driver name with current generator
         self._set_name()
 
+        # Add all design variables
+        dv_meta = self._designvars
+        self._indep_list = list(dv_meta)
+
+        # Add all objectives
+        objs = self.get_objective_values()
+        for name in objs:
+            self._quantities.append(name)
+
+        # Add all constraints
+        con_meta = self._cons
+        for name, _ in con_meta.items():
+            self._quantities.append(name)
+
         if MPI and self.options['run_parallel']:
             case_gen = self._parallel_generator
         else:
@@ -192,7 +216,7 @@ class DOEDriver(Driver):
                 msg = "Error assigning %s = %s: " % (dv_name, dv_val) + str(err)
             finally:
                 if msg:
-                    raise(ValueError(msg))
+                    raise ValueError(msg)
 
         with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
             try:
@@ -210,6 +234,12 @@ class DOEDriver(Driver):
             # save reference to metadata for use in record_iteration
             self._metadata = metadata
 
+        opts = self.recording_options
+        if opts['record_derivatives']:
+            self._compute_totals(of=self._quantities,
+                                 wrt=self._indep_list,
+                                 return_format=self._total_jac_format)
+
     def _parallel_generator(self, design_vars, model=None):
         """
         Generate case for this processor when running under MPI.
