@@ -1,12 +1,10 @@
 """Define the base Jacobian class."""
 import weakref
-import sys
 
 import numpy as np
-from numpy.random import rand
 
-from collections import OrderedDict, defaultdict
-from scipy.sparse import issparse, coo_matrix
+from collections import defaultdict
+from scipy.sparse import issparse
 
 from openmdao.core.constants import INT_DTYPE
 from openmdao.utils.name_maps import key2abs_key, rel_name2abs_name
@@ -44,8 +42,9 @@ class Jacobian(object):
         When True, this Jacobian is under complex step, using a complex jacobian.
     _abs_keys : defaultdict
         A cache dict for key to absolute key.
-    _randomize : bool
-        If True, sparsity is being computed for simultaneous derivative coloring.
+    _randgen : Generator or None
+        If not None, use the generator to generate random numbers during computation of
+        sparsity for for simultaneous derivative coloring.
     _col_var_offset : dict
         Maps column name to offset into the result array.
     _col_varnames : list
@@ -62,7 +61,7 @@ class Jacobian(object):
         self._subjacs_info = system._subjacs_info
         self._under_complex_step = False
         self._abs_keys = defaultdict(bool)
-        self._randomize = False
+        self._randgen = None
         self._col_var_offset = None
         self._col_varnames = None
         self._col2name_ind = None
@@ -275,7 +274,7 @@ class Jacobian(object):
         """
         if isinstance(subjac, sparse_types):  # sparse
             sparse = subjac.copy()
-            sparse.data = rand(sparse.data.size)
+            sparse.data = self._randgen.random(sparse.data.size)
             sparse.data += 1.0
             return sparse
 
@@ -290,11 +289,11 @@ class Jacobian(object):
             assert subjac_info['rows'] is None
             rows, cols, shape = subjac_info['sparsity']
             r = np.zeros(shape)
-            val = rand(len(rows))
+            val = self._randgen.random(len(rows))
             val += 1.0
             r[rows, cols] = val
         else:
-            r = rand(*subjac.shape)
+            r = self._randgen.random(subjac.shape)
             r += 1.0
         return r
 
@@ -423,6 +422,35 @@ class Jacobian(object):
                     if match_inds.size > 0:
                         subjac['val'][match_inds] = column[start:end][subjac['rows'][match_inds]]
 
+    def set_dense_jac(self, system, jac):
+        """
+        Assign a dense jacobian to this jacobian.
+
+        This assumes that any column does not attempt to set any nonzero values that are
+        outside of specified sparsity patterns for any of the subjacs.
+
+        Parameters
+        ----------
+        system : System
+            The system that owns this jacobian.
+        jac : ndarray
+            Dense jacobian.
+        """
+        if self._col_varnames is None:
+            self._setup_index_maps(system)
+
+        wrtiter = list(system._jac_wrt_iter())
+        for of, start, end, _, _ in system._jac_of_iter():
+            for wrt, wstart, wend, _, _, _ in wrtiter:
+                key = (of, wrt)
+                if key in self._subjacs_info:
+                    subjac = self._subjacs_info[key]
+                    if subjac['cols'] is None:  # dense
+                        subjac['val'][:, :] = jac[start:end, wstart:wend]
+                    else:  # our COO format
+                        subj = jac[start:end, wstart:wend]
+                        subjac['val'][:] = subj[subjac['rows'], subjac['cols']]
+
     def _restore_approx_sparsity(self):
         """
         Revert all subjacs back to the way they were as declared by the user.
