@@ -14,8 +14,9 @@ from openmdao.recorders.sqlite_recorder import blob_to_array
 from openmdao.utils.record_util import deserialize, get_source_system
 from openmdao.utils.variable_table import write_var_table
 from openmdao.utils.general_utils import make_set, match_prom_or_abs
-from openmdao.utils.units import unit_conversion
+from openmdao.utils.units import unit_conversion, simplify_unit
 from openmdao.recorders.sqlite_recorder import format_version as current_version
+from openmdao.utils.om_warnings import issue_warning
 
 _AMBIGOUS_PROM_NAME = object()
 
@@ -24,6 +25,29 @@ class Case(object):
     """
     Case wraps the data from a single iteration of a recording to make it more easily accessible.
 
+    Parameters
+    ----------
+    source : str
+        The unique id of the system/solver/driver/problem that did the recording.
+    data : dict-like
+        Dictionary of data for a case.
+    prom2abs : {'input': dict, 'output': dict}
+        Dictionary mapping promoted names of all variables to absolute names.
+    abs2prom : {'input': dict, 'output': dict}
+        Dictionary mapping absolute names of all variables to promoted names.
+    abs2meta : dict
+        Dictionary mapping absolute names of all variables to variable metadata.
+    conns : dict
+        Dictionary of all model connections.
+    auto_ivc_map : dict
+        Dictionary that maps all auto_ivc sources to either an absolute input name for single
+        connections or a promoted input name for multiple connections. This is for output
+        display.
+    var_info : dict
+        Dictionary with information about variables (scaling, indices, execution order).
+    data_format : int
+        A version number specifying the format of array data, if not numpy arrays.
+
     Attributes
     ----------
     source : str
@@ -75,29 +99,6 @@ class Case(object):
                  data_format=None):
         """
         Initialize.
-
-        Parameters
-        ----------
-        source : str
-            The unique id of the system/solver/driver/problem that did the recording.
-        data : dict-like
-            Dictionary of data for a case
-        prom2abs : {'input': dict, 'output': dict}
-            Dictionary mapping promoted names of all variables to absolute names.
-        abs2prom : {'input': dict, 'output': dict}
-            Dictionary mapping absolute names of all variables to promoted names.
-        abs2meta : dict
-            Dictionary mapping absolute names of all variables to variable metadata.
-        conns : dict
-            Dictionary of all model connections.
-        auto_ivc_map : dict
-            Dictionary that maps all auto_ivc sources to either an absolute input name for single
-            connections or a promoted input name for multiple connections. This is for output
-            display.
-        var_info : dict
-            Dictionary with information about variables (scaling, indices, execution order).
-        data_format : int
-            A version number specifying the format of array data, if not numpy arrays.
         """
         self.source = source
         self._format_version = data_format
@@ -123,7 +124,7 @@ class Case(object):
 
         # for a solver or problem case
         self.abs_err = data['abs_err'] if 'abs_err' in data.keys() else None
-        self.rel_err = data['abs_err'] if 'rel_err' in data.keys() else None
+        self.rel_err = data['rel_err'] if 'rel_err' in data.keys() else None
 
         # rename solver keys
         if 'solver_inputs' in data.keys():
@@ -266,16 +267,17 @@ class Case(object):
 
         if units is not None:
             base_units = self._get_units(name)
+            simp_units = simplify_unit(units)
 
             if base_units is None:
                 msg = "Can't express variable '{}' with units of 'None' in units of '{}'."
-                raise TypeError(msg.format(name, units))
+                raise TypeError(msg.format(name, simp_units))
 
             try:
-                scale, offset = unit_conversion(base_units, units)
+                scale, offset = unit_conversion(base_units, simp_units)
             except TypeError:
                 msg = "Can't express variable '{}' with units of '{}' in units of '{}'."
-                raise TypeError(msg.format(name, base_units, units))
+                raise TypeError(msg.format(name, base_units, simp_units))
 
             val = (val + offset) * scale
 
@@ -391,7 +393,7 @@ class Case(object):
         return self._get_variables_of_type('response', scaled, use_indices)
 
     def list_inputs(self,
-                    values=True,
+                    val=True,
                     prom_name=False,
                     units=False,
                     shape=False,
@@ -401,13 +403,16 @@ class Case(object):
                     tags=None,
                     includes=None,
                     excludes=None,
-                    out_stream=_DEFAULT_OUT_STREAM):
+                    out_stream=_DEFAULT_OUT_STREAM,
+                    values=None,
+                    print_min=False,
+                    print_max=False):
         """
         Return and optionally log a list of input names and other optional information.
 
         Parameters
         ----------
-        values : bool, optional
+        val : bool, optional
             When True, display/return input values. Default is True.
         prom_name : bool, optional
             When True, display/return the promoted name of the variable.
@@ -430,28 +435,56 @@ class Case(object):
             User defined tags that can be used to filter what gets listed. Only inputs with the
             given tags will be listed.
             Default is None, which means there will be no filtering based on tags.
-        includes : iter of str or None
+        includes : str, iter of str, or None
             Glob patterns for pathnames to include in the check. Default is None, which
             includes all.
-        excludes : iter of str or None
+        excludes : str, iter of str, or None
             Glob patterns for pathnames to exclude from the check. Default is None, which
             excludes nothing.
         out_stream : file-like object
             Where to send human readable output. Default is sys.stdout.
             Set to None to suppress.
+        values : bool, optional
+            This argument has been deprecated and will be removed in 4.0.
+        print_min : bool, optional
+            When true, if the input value is an array, print its smallest value.
+        print_max : bool, optional
+            When true, if the input value is an array, print its largest value.
 
         Returns
         -------
         list
-            list of input names and other optional information about those inputs
+            List of input names and other optional information about those inputs.
         """
-        meta = self._abs2meta
+        abs2meta = self._abs2meta
         inputs = []
 
+        # string to display when an attribute is not available (e.g. for a discrete)
+        NA = 'Unavailable'
+
+        if values is not None:
+            issue_warning("'value' is deprecated and will be removed in 4.0. "
+                          "Please index in using 'val'")
+        elif not val and values:
+            values = True
+        else:
+            values = val
+
+        if isinstance(includes, str):
+            includes = [includes, ]
+
+        if isinstance(excludes, str):
+            excludes = [excludes, ]
+
         if self.inputs is not None:
+            print_options = np.get_printoptions()
+            np_precision = print_options['precision']
+
             for var_name in self.inputs.absolute_names():
+                meta = abs2meta[var_name]
+
                 # Filter based on tags
-                if tags and not (make_set(tags) & make_set(meta[var_name]['tags'])):
+                if tags and not (make_set(tags) & make_set(meta['tags'])):
                     continue
 
                 var_name_prom = self._abs2prom['input'][var_name]
@@ -463,33 +496,42 @@ class Case(object):
 
                 var_meta = {}
                 if values:
+                    var_meta['val'] = val
                     var_meta['value'] = val
+                    if isinstance(val, np.ndarray):
+                        if print_min:
+                            var_meta['min'] = np.round(np.min(val), np_precision)
+
+                        if print_max:
+                            var_meta['max'] = np.round(np.max(val), np_precision)
+
                 if prom_name:
                     var_meta['prom_name'] = var_name_prom
                 if units:
-                    var_meta['units'] = meta[var_name]['units']
+                    var_meta['units'] = meta.get('units', NA)
                 if shape:
-                    var_meta['shape'] = val.shape
+                    try:
+                        var_meta['shape'] = val.shape
+                    except AttributeError:
+                        var_meta['shape'] = NA
                 if desc:
-                    var_meta['desc'] = meta[var_name]['desc']
+                    var_meta['desc'] = meta['desc']
 
                 inputs.append((var_name, var_meta))
 
-        if out_stream is _DEFAULT_OUT_STREAM:
-            out_stream = sys.stdout
-
         if out_stream:
-            if self.inputs is None or len(self.inputs) == 0:
-                out_stream.write('WARNING: Inputs not recorded. Make sure your recording ' +
-                                 'settings have record_inputs set to True\n')
-
-            self._write_table('input', inputs, hierarchical, print_arrays, out_stream)
+            if self.inputs:
+                self._write_table('input', inputs, hierarchical, print_arrays, out_stream)
+            else:
+                ostream = sys.stdout if out_stream is _DEFAULT_OUT_STREAM else out_stream
+                ostream.write('WARNING: Inputs not recorded. Make sure your recording ' +
+                              'settings have record_inputs set to True\n')
 
         return inputs
 
     def list_outputs(self,
                      explicit=True, implicit=True,
-                     values=True,
+                     val=True,
                      prom_name=False,
                      residuals=False,
                      residuals_tol=None,
@@ -504,17 +546,20 @@ class Case(object):
                      includes=None,
                      excludes=None,
                      list_autoivcs=False,
-                     out_stream=_DEFAULT_OUT_STREAM):
+                     out_stream=_DEFAULT_OUT_STREAM,
+                     values=None,
+                     print_min=False,
+                     print_max=False):
         """
         Return and optionally log a list of output names and other optional information.
 
         Parameters
         ----------
         explicit : bool, optional
-            include outputs from explicit components. Default is True.
+            Include outputs from explicit components. Default is True.
         implicit : bool, optional
-            include outputs from implicit components. Default is True.
-        values : bool, optional
+            Include outputs from implicit components. Default is True.
+        val : bool, optional
             When True, display/return output values. Default is True.
         prom_name : bool, optional
             When True, display/return the promoted name of the variable.
@@ -547,10 +592,10 @@ class Case(object):
             User defined tags that can be used to filter what gets listed. Only outputs with the
             given tags will be listed.
             Default is None, which means there will be no filtering based on tags.
-        includes : iter of str or None
+        includes : str, iter of str, or None
             Glob patterns for pathnames to include in the check. Default is None, which
             includes all.
-        excludes : iter of str or None
+        excludes : str, iter of str, or None
             Glob patterns for pathnames to exclude from the check. Default is None, which
             excludes nothing.
         list_autoivcs : bool
@@ -558,22 +603,50 @@ class Case(object):
         out_stream : file-like
             Where to send human readable output. Default is sys.stdout.
             Set to None to suppress.
+        values : bool, optional
+            This argument has been deprecated and will be removed in 4.0.
+        print_min : bool, optional
+            When true, if the output value is an array, print its smallest value.
+        print_max : bool, optional
+            When true, if the output value is an array, print its largest value.
 
         Returns
         -------
         list
-            list of output names and other optional information about those outputs
+            List of output names and other optional information about those outputs.
         """
-        meta = self._abs2meta
+        abs2meta = self._abs2meta
         expl_outputs = []
         impl_outputs = []
 
+        # string to display when an attribute is not available (e.g. for a discrete)
+        NA = 'Unavailable'
+
+        if values is not None:
+            issue_warning("'value' is deprecated and will be removed in 4.0. "
+                          "Please index in using 'val'")
+        elif not val and values:
+            values = True
+        else:
+            values = val
+
+        if isinstance(includes, str):
+            includes = [includes, ]
+
+        if isinstance(excludes, str):
+            excludes = [excludes, ]
+
+        print_options = np.get_printoptions()
+        np_precision = print_options['precision']
+
         for var_name in self.outputs.absolute_names():
             if not list_autoivcs and var_name.startswith('_auto_ivc.'):
                 continue
 
+            meta = abs2meta[var_name]
+
             # Filter based on tags
-            if tags and not (make_set(tags) & make_set(meta[var_name]['tags'])):
+            if tags and not (make_set(tags) & make_set(meta['tags'])):
                 continue
 
             var_name_prom = self._abs2prom['output'][var_name]
@@ -593,36 +666,44 @@ class Case(object):
 
             var_meta = {}
             if values:
+                var_meta['val'] = val
                 var_meta['value'] = val
+                if isinstance(val, np.ndarray):
+                    if print_min:
+                        var_meta['min'] = np.round(np.min(val), np_precision)
+
+                    if print_max:
+                        var_meta['max'] = np.round(np.max(val), np_precision)
             if prom_name:
                 var_meta['prom_name'] = var_name_prom
             if residuals:
                 var_meta['resids'] = resids
             if units:
-                var_meta['units'] = meta[var_name]['units']
+                var_meta['units'] = meta.get('units', NA)
             if shape:
-                var_meta['shape'] = val.shape
+                try:
+                    var_meta['shape'] = val.shape
+                except AttributeError:
+                    var_meta['shape'] = NA
             if bounds:
-                var_meta['lower'] = meta[var_name]['lower']
-                var_meta['upper'] = meta[var_name]['upper']
+                var_meta['lower'] = meta.get('lower', NA)
+                var_meta['upper'] = meta.get('upper', NA)
             if scaling:
-                var_meta['ref'] = meta[var_name]['ref']
-                var_meta['ref0'] = meta[var_name]['ref0']
-                var_meta['res_ref'] = meta[var_name]['res_ref']
+                var_meta['ref'] = meta.get('ref', NA)
+                var_meta['ref0'] = meta.get('ref0', NA)
+                var_meta['res_ref'] = meta.get('res_ref', NA)
             if desc:
-                var_meta['desc'] = meta[var_name]['desc']
-            if meta[var_name]['explicit']:
+                var_meta['desc'] = meta['desc']
+            if meta['explicit']:
                 expl_outputs.append((var_name, var_meta))
             else:
                 impl_outputs.append((var_name, var_meta))
 
-        if out_stream is _DEFAULT_OUT_STREAM:
-            out_stream = sys.stdout
-
         if out_stream:
-            if self.outputs is None or len(self.outputs) == 0:
-                out_stream.write('WARNING: Outputs not recorded. Make sure your recording ' +
-                                 'settings have record_outputs set to True\n')
+            if not self.outputs:
+                ostream = sys.stdout if out_stream is _DEFAULT_OUT_STREAM else out_stream
+                ostream.write('WARNING: Outputs not recorded. Make sure your recording ' +
+                              'settings have record_outputs set to True\n')
             if explicit:
                 self._write_table('explicit', expl_outputs, hierarchical, print_arrays, out_stream)
             if implicit:
@@ -733,7 +814,7 @@ class Case(object):
                     return_name = auto_ivc_map[name]
                 else:
                     return_name = name
-                ret_vars[return_name] = val = self.outputs[name]
+                ret_vars[return_name] = val = self.outputs[name].copy()
                 if update_vals and name in self._var_info:
                     meta = self._var_info[name]
                     if use_indices and meta['indices'] is not None:
@@ -753,6 +834,23 @@ class PromAbsDict(dict):
     """
     A dictionary that enables accessing values via absolute or promoted variable names.
 
+    Parameters
+    ----------
+    values : array or dict
+        Numpy structured array or dictionary of values.
+    prom2abs : dict
+        Dictionary mapping promoted names to absolute names.
+    abs2prom : dict
+        Dictionary mapping absolute names in the output vector to promoted names.
+    data_format : int
+        A version number specifying the OpenMDAO SQL case database version.
+    in_prom2abs : dict
+        Dictionary mapping promoted names in the input vector to absolute names.
+    auto_ivc_map : dict
+        Dictionary that maps all auto_ivc sources to either an absolute input name for single
+        connections or a promoted input name for multiple connections. This is for output
+        display.
+
     Attributes
     ----------
     _values : array or dict
@@ -774,23 +872,6 @@ class PromAbsDict(dict):
                  in_prom2abs=None, auto_ivc_map=None):
         """
         Initialize.
-
-        Parameters
-        ----------
-        values : array or dict
-            Numpy structured array or dictionary of values.
-        prom2abs : dict
-            Dictionary mapping promoted names to absolute names.
-        abs2prom : dict
-            Dictionary mapping absolute names in the output vector to promoted names.
-        data_format : int
-            A version number specifying the OpenMDAO SQL case database version.
-        in_prom2abs : dict
-           Dictionary mapping promoted names in the input vector to absolute names.
-        auto_ivc_map : dict
-            Dictionary that maps all auto_ivc sources to either an absolute input name for single
-            connections or a promoted input name for multiple connections. This is for output
-            display.
         """
         super().__init__()
 
@@ -924,7 +1005,7 @@ class PromAbsDict(dict):
 
         Parameters
         ----------
-        key : string
+        key : str
             Absolute or promoted variable name.
 
         Returns
@@ -966,7 +1047,7 @@ class PromAbsDict(dict):
 
         Parameters
         ----------
-        key : string
+        key : str
             Absolute or promoted variable name.
         value : any
             value for variable
