@@ -192,13 +192,13 @@ class SellarNoDerivatives(om.Group):
     """
 
     def initialize(self):
-        self.options.declare('nonlinear_solver', default=om.NonlinearBlockGS,
+        self.options.declare('nonlinear_solver', default=None,
                              desc='Nonlinear solver for Sellar MDA')
         self.options.declare('nl_atol', default=None,
                              desc='User-specified atol for nonlinear solver.')
         self.options.declare('nl_maxiter', default=None,
                              desc='Iteration limit for nonlinear solver.')
-        self.options.declare('linear_solver', default=om.ScipyKrylov,
+        self.options.declare('linear_solver', default=None,
                              desc='Linear solver')
         self.options.declare('ln_atol', default=None,
                              desc='User-specified atol for linear solver.')
@@ -221,7 +221,8 @@ class SellarNoDerivatives(om.Group):
         self.set_input_defaults('z', np.array([5.0, 2.0]))
 
         nl = self.options['nonlinear_solver']
-        self.nonlinear_solver = nl() if inspect.isclass(nl) else nl
+        if nl:
+            self.nonlinear_solver = nl() if inspect.isclass(nl) else nl
         if self.options['nl_atol']:
             self.nonlinear_solver.options['atol'] = self.options['nl_atol']
         if self.options['nl_maxiter']:
@@ -229,7 +230,8 @@ class SellarNoDerivatives(om.Group):
 
     def configure(self):
         ln = self.options['linear_solver']
-        self.cycle.linear_solver = ln() if inspect.isclass(ln) else ln
+        if ln:
+            self.cycle.linear_solver = ln() if inspect.isclass(ln) else ln
         if self.options['ln_atol']:
             self.cycle.linear_solver.options['atol'] = self.options['ln_atol']
         if self.options['ln_maxiter']:
@@ -242,13 +244,13 @@ class SellarDerivatives(om.Group):
     """
 
     def initialize(self):
-        self.options.declare('nonlinear_solver', default=om.NonlinearBlockGS,
+        self.options.declare('nonlinear_solver', default=None,
                              desc='Nonlinear solver (class or instance) for Sellar MDA')
         self.options.declare('nl_atol', default=None,
                              desc='User-specified atol for nonlinear solver.')
         self.options.declare('nl_maxiter', default=None,
                              desc='Iteration limit for nonlinear solver.')
-        self.options.declare('linear_solver', default=om.ScipyKrylov,
+        self.options.declare('linear_solver', default=None,
                              desc='Linear solver (class or instance)')
         self.options.declare('ln_atol', default=None,
                              desc='User-specified atol for linear solver.')
@@ -272,64 +274,18 @@ class SellarDerivatives(om.Group):
         self.set_input_defaults('z', np.array([5.0, 2.0]))
 
         nl = self.options['nonlinear_solver']
-        self.nonlinear_solver = nl() if inspect.isclass(nl) else nl
+        if nl:
+            self.nonlinear_solver = nl() if inspect.isclass(nl) else nl
+
         if self.options['nl_atol']:
             self.nonlinear_solver.options['atol'] = self.options['nl_atol']
         if self.options['nl_maxiter']:
             self.nonlinear_solver.options['maxiter'] = self.options['nl_maxiter']
 
         ln = self.options['linear_solver']
-        self.linear_solver = ln() if inspect.isclass(ln) else ln
-        if self.options['ln_atol']:
-            self.linear_solver.options['atol'] = self.options['ln_atol']
-        if self.options['ln_maxiter']:
-            self.linear_solver.options['maxiter'] = self.options['ln_maxiter']
+        if ln:
+            self.linear_solver = ln() if inspect.isclass(ln) else ln
 
-
-class SellarDerivativesPreAutoIVC(om.Group):
-    """
-    Group containing the Sellar MDA. This version uses the disciplines with derivatives.
-    """
-
-    def initialize(self):
-        self.options.declare('nonlinear_solver', default=om.NonlinearBlockGS,
-                             desc='Nonlinear solver (class or instance) for Sellar MDA')
-        self.options.declare('nl_atol', default=None,
-                             desc='User-specified atol for nonlinear solver.')
-        self.options.declare('nl_maxiter', default=None,
-                             desc='Iteration limit for nonlinear solver.')
-        self.options.declare('linear_solver', default=om.ScipyKrylov,
-                             desc='Linear solver (class or instance)')
-        self.options.declare('ln_atol', default=None,
-                             desc='User-specified atol for linear solver.')
-        self.options.declare('ln_maxiter', default=None,
-                             desc='Iteration limit for linear solver.')
-
-    def setup(self):
-        self.add_subsystem('d1', SellarDis1withDerivatives(), promotes=['x', 'z', 'y1', 'y2'])
-        self.add_subsystem('d2', SellarDis2withDerivatives(), promotes=['z', 'y1', 'y2'])
-
-        self.add_subsystem('obj_cmp', om.ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)', obj=0.0,
-                                                  x=0.0, z=np.array([0.0, 0.0]), y1=0.0, y2=0.0),
-                           promotes=['obj', 'x', 'z', 'y1', 'y2'])
-
-        self.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1', con1=0.0, y1=0.0),
-                           promotes=['con1', 'y1'])
-        self.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0', con2=0.0, y2=0.0),
-                           promotes=['con2', 'y2'])
-
-        self.set_input_defaults('x', 1.0)
-        self.set_input_defaults('z', np.array([5.0, 2.0]))
-
-        nl = self.options['nonlinear_solver']
-        self.nonlinear_solver = nl() if inspect.isclass(nl) else nl
-        if self.options['nl_atol']:
-            self.nonlinear_solver.options['atol'] = self.options['nl_atol']
-        if self.options['nl_maxiter']:
-            self.nonlinear_solver.options['maxiter'] = self.options['nl_maxiter']
-
-        ln = self.options['linear_solver']
-        self.linear_solver = ln() if inspect.isclass(ln) else ln
         if self.options['ln_atol']:
             self.linear_solver.options['atol'] = self.options['ln_atol']
         if self.options['ln_maxiter']:
@@ -358,9 +314,6 @@ class SellarDerivativesConnected(om.Group):
         self.set_input_defaults('x', 1.0)
         self.set_input_defaults('z', np.array([5.0, 2.0]))
 
-        self.nonlinear_solver = om.NonlinearBlockGS()
-        self.linear_solver = om.ScipyKrylov()
-
 
 class SellarDerivativesGrouped(om.Group):
     """
@@ -368,21 +321,21 @@ class SellarDerivativesGrouped(om.Group):
     """
 
     def initialize(self):
-        self.options.declare('nonlinear_solver', default=om.NonlinearBlockGS, recordable=False,
+        self.options.declare('nonlinear_solver', default=None, recordable=False,
                              desc='Nonlinear solver (class or instance) for Sellar MDA')
         self.options.declare('nl_atol', default=None,
                              desc='User-specified atol for nonlinear solver.')
         self.options.declare('nl_maxiter', default=None,
                              desc='Iteration limit for nonlinear solver.')
-        self.options.declare('linear_solver', default=om.ScipyKrylov, recordable=False,
+        self.options.declare('linear_solver', default=None, recordable=False,
                              desc='Linear solver (class or instance)')
         self.options.declare('ln_atol', default=None,
                              desc='User-specified atol for linear solver.')
         self.options.declare('ln_maxiter', default=None,
                              desc='Iteration limit for linear solver.')
-        self.options.declare('mda_nonlinear_solver', default=om.NonlinearBlockGS, recordable=False,
+        self.options.declare('mda_nonlinear_solver', default=None, recordable=False,
                              desc='Nonlinear solver (class or instance)')
-        self.options.declare('mda_linear_solver', default=om.ScipyKrylov, recordable=False,
+        self.options.declare('mda_linear_solver', default=None, recordable=False,
                              desc='Linear solver (class or instance) for Sellar MDA')
 
     def setup(self):
@@ -401,24 +354,30 @@ class SellarDerivativesGrouped(om.Group):
         self.set_input_defaults('z', np.array([5.0, 2.0]))
 
         nl = self.options['nonlinear_solver']
-        self.nonlinear_solver = nl() if inspect.isclass(nl) else nl
+        if nl:
+            self.nonlinear_solver = nl() if inspect.isclass(nl) else nl
+
         if self.options['nl_atol']:
             self.nonlinear_solver.options['atol'] = self.options['nl_atol']
         if self.options['nl_maxiter']:
             self.nonlinear_solver.options['maxiter'] = self.options['nl_maxiter']
 
         ln = self.options['linear_solver']
-        self.linear_solver = ln() if inspect.isclass(ln) else ln
+        if ln:
+            self.linear_solver = ln() if inspect.isclass(ln) else ln
+
         if self.options['ln_atol']:
             self.linear_solver.options['atol'] = self.options['ln_atol']
         if self.options['ln_maxiter']:
             self.linear_solver.options['maxiter'] = self.options['ln_maxiter']
 
         nl = self.options['mda_nonlinear_solver']
-        self.mda.nonlinear_solver = nl() if inspect.isclass(nl) else nl
+        if nl:
+            self.mda.nonlinear_solver = nl() if inspect.isclass(nl) else nl
 
         ln = self.options['mda_linear_solver']
-        self.mda.linear_solver = ln() if inspect.isclass(ln) else ln
+        if ln:
+            self.mda.linear_solver = ln() if inspect.isclass(ln) else ln
 
 
 class StateConnection(om.ImplicitComponent):
