@@ -7,6 +7,8 @@ import inspect
 import warnings
 import sys
 
+from openmdao.utils.om_warnings import issue_warning
+
 
 # global dict of hooks
 # {class_name: { inst_id: {fname: [pre_hooks, post_hooks]}}}
@@ -200,8 +202,6 @@ def _register_hook(fname, class_name, inst_id=None, pre=None, post=None, ncalls=
 
     Parameters
     ----------
-    hookfunc : function
-        A function to execute in the designated hook location.
     fname : str
         The name of the function where the pre and/or post hook will be applied.
     class_name : str
@@ -264,8 +264,8 @@ def _remove_hook(to_remove, hooks, class_name, fname, hook_loc, inst_id):
                     hooks.remove(hook)
                     break
             else:
-                raise RuntimeError(f"Couldn't find the given '{hook_loc}' function in the "
-                                   f"{hook_loc} hooks for {class_name}.{fname}.")
+                issue_warning(f"Couldn't find the given '{hook_loc}' function in the "
+                              f"{hook_loc} hooks for {class_name}.{fname}.")
 
 
 def _unregister_hook(fname, class_name, inst_id=None, pre=True, post=True):
@@ -317,6 +317,9 @@ def _unregister_hook(fname, class_name, inst_id=None, pre=True, post=True):
 
     if todel:
         for name in todel:
-            del classhooks[name]
+            try:
+                del classhooks[name]
+            except KeyError:
+                pass
         if not classhooks:  # removed last entry for this class
             del _hooks[class_name]
