@@ -3,23 +3,23 @@ import unittest
 import pathlib
 import sys
 import os
-import functools
 from io import StringIO
 
 import openmdao.api as om
 from openmdao.test_suite.components.paraboloid import Paraboloid
-from openmdao.test_suite.components.sellar_feature import SellarMDA
-import openmdao.core.problem
+from openmdao.test_suite.groups.parallel_groups import Diamond
+from openmdao.core.problem import _default_prob_name
+import openmdao.core.problem as probmod
 from openmdao.core.constants import _UNDEFINED
-from openmdao.utils.assert_utils import assert_warning
 from openmdao.utils.general_utils import set_pyoptsparse_opt
 from openmdao.utils.reports_system import set_reports_dir, _reports_dir, register_report, \
-    list_reports, clear_reports, get_reports_dir, _reset_reports_dir, activate_report
+    list_reports, clear_reports, _reset_reports_dir, activate_report
 from openmdao.utils.testing_utils import use_tempdirs
 from openmdao.utils.mpi import MPI
 from openmdao.utils.tests.test_hooks import hooks_active
 from openmdao.visualization.n2_viewer.n2_viewer import _default_n2_filename, _run_n2_report
 from openmdao.visualization.scaling_viewer.scaling_report import _default_scaling_filename
+from openmdao.visualization.opt_report.opt_report import _default_optimizer_report_filename
 
 try:
     from openmdao.vectors.petsc_vector import PETScVector
@@ -37,9 +37,10 @@ class TestReportsSystem(unittest.TestCase):
     def setUp(self):
         self.n2_filename = _default_n2_filename
         self.scaling_filename = _default_scaling_filename
+        self.optimizer_filename = _default_optimizer_report_filename
 
         # set things to a known initial state for all the test runs
-        openmdao.core.problem._problem_names = []  # need to reset these to simulate separate runs
+        probmod._clear_problem_names()  # need to reset these to simulate separate runs
         os.environ.pop('OPENMDAO_REPORTS', None)
         os.environ.pop('OPENMDAO_REPORTS_DIR', None)
         # We need to remove the TESTFLO_RUNNING environment variable for these tests to run.
@@ -113,9 +114,7 @@ class TestReportsSystem(unittest.TestCase):
         prob.setup()
         prob.run_model()  # need to do run_model in this test so sub problem is created
 
-        probname = prob._name
-        subprobname = G.nonlinear_solver._problem._name
-        return probname, subprobname
+        return prob._name, G.nonlinear_solver._problem._name
 
     @hooks_active
     def test_report_generation_basic(self):
@@ -128,6 +127,8 @@ class TestReportsSystem(unittest.TestCase):
         self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertTrue(path.is_file(), f'The scaling report file, {str(path)}, was not found')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertTrue(path.is_file(), f'The optimizer report file, {str(path)}, was not found')
 
     @hooks_active
     @unittest.skipUnless(OPTIMIZER, "This test requires pyOptSparseDriver.")
@@ -142,6 +143,8 @@ class TestReportsSystem(unittest.TestCase):
         self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertTrue(path.is_file(), f'The scaling report file, {str(path)}, was not found')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertTrue(path.is_file(), f'The optimizer report file, {str(path)}, was not found')
 
     @hooks_active
     def test_report_generation_basic_doedriver(self):
@@ -152,10 +155,13 @@ class TestReportsSystem(unittest.TestCase):
 
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
-        # DOEDriver won't cause the creation of a scaling report
+        # DOEDriver won't cause the creation of a scaling or optimizer report
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_list_reports(self):
@@ -172,6 +178,8 @@ class TestReportsSystem(unittest.TestCase):
                         '"N2 diagram" expected in list_reports output but was not found')
         self.assertTrue('Driver scaling report' in output,
                         '"Driver scaling report" expected in list_reports output but was not found')
+        self.assertTrue('Summary of optimization' in output,
+                        '"Summary of optimization" expected in list_reports output but was not found')
 
     @hooks_active
     def test_report_generation_no_reports_using_env_var(self):
@@ -186,10 +194,13 @@ class TestReportsSystem(unittest.TestCase):
 
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The N2 report file, {str(path)} was found but should not have')
+                         f'The N2 report file, {str(path)} was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_selected_reports_using_env_var(self):
@@ -206,7 +217,10 @@ class TestReportsSystem(unittest.TestCase):
         self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_selected_reports_override_env_var(self):
@@ -214,19 +228,22 @@ class TestReportsSystem(unittest.TestCase):
         os.environ['OPENMDAO_REPORTS'] = 'n2'
         clear_reports()
 
-        prob = self.setup_and_run_simple_problem(reports=['n2', 'scaling'])
+        prob = self.setup_and_run_simple_problem(reports=['optimizer', 'scaling'])
 
         # See if the report files exist and if they have the right names
         problem_reports_dir = pathlib.Path(_reports_dir).joinpath(prob._name)
 
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
-        self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
+        self.assertFalse(path.is_file(),
+                         f'The N2 report file, {str(path)} was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertTrue(path.is_file(), f'The scaling report file, {str(path)} was not found')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertTrue(path.is_file(), f'The optimizer report file, {str(path)}, was not found')
 
     @hooks_active
     def test_report_generation_selected_reports_override_env_var2(self):
-        # test use of problem reports to override OPENMDAO_REPORTS
+        # test use of problem reports to override OPENMDAO_REPORTS. This time with two reports
         os.environ['OPENMDAO_REPORTS'] = 'n2,scaling'
         clear_reports()
 
@@ -237,10 +254,13 @@ class TestReportsSystem(unittest.TestCase):
 
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The N2 report file, {str(path)}, was found but should not have')
+                         f'The N2 report file, {str(path)}, was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_set_reports_dir_using_env_var(self):
@@ -259,6 +279,8 @@ class TestReportsSystem(unittest.TestCase):
         self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertTrue(path.is_file(), f'The scaling report file, {str(path)}, was not found')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertTrue(path.is_file(), f'The optimizer report file, {str(path)}, was not found')
 
     @hooks_active
     def test_report_generation_user_defined_report(self):
@@ -319,7 +341,7 @@ class TestReportsSystem(unittest.TestCase):
     def test_report_generation_multiple_problems(self):
         probname, subprobname = self.setup_and_run_model_with_subproblem()
 
-        # The multiple problem code only runs model so no scaling reports to look for
+        # The multiple problem code only runs model so no scaling or optimizer reports to look for
         for problem_name in [probname, subprobname]:
             problem_reports_dir = pathlib.Path(_reports_dir).joinpath(f'{problem_name}')
             path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
@@ -329,17 +351,18 @@ class TestReportsSystem(unittest.TestCase):
     def test_report_generation_multiple_problems_report_specific_problem(self):
         # test the ability to register a report with a specific Problem name rather
         #   than have the report run for all Problems
-
         os.environ['OPENMDAO_REPORTS'] = 'n2_report'
 
         # to simplify things, just do n2.
         clear_reports()
         register_report("n2_report", _run_n2_report, 'N2 diagram', 'Problem', 'final_setup', 'post',
                         self.n2_filename,
-                        inst_id='problem2')
+                        inst_id=_default_prob_name() + '2')
 
         probname, subprobname = self.setup_and_run_model_with_subproblem()
 
+        self.assertEqual(_default_prob_name(), probname)
+
         # The multiple problem code only runs model so no scaling reports to look for
         problem_reports_dir = pathlib.Path(_reports_dir).joinpath(f'{subprobname}')
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
@@ -350,7 +373,7 @@ class TestReportsSystem(unittest.TestCase):
         problem_reports_dir = pathlib.Path(_reports_dir).joinpath(f'{probname}')
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The N2 report file, {str(path)} was found but should not have')
+                         f'The N2 report file, {str(path)} was found but should not exist.')
 
     @hooks_active
     def test_report_generation_test_TESTFLO_RUNNING(self):
@@ -365,10 +388,13 @@ class TestReportsSystem(unittest.TestCase):
 
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The N2 report file, {str(path)} was found but should not have')
+                         f'The N2 report file, {str(path)} was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_basic_problem_reports_argument_false(self):
@@ -378,10 +404,13 @@ class TestReportsSystem(unittest.TestCase):
         problem_reports_dir = pathlib.Path(_reports_dir).joinpath(prob._name)
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The N2 report file, {str(path)} was found but should not have')
+                         f'The N2 report file, {str(path)} was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_basic_problem_reports_argument_none(self):
@@ -392,10 +421,13 @@ class TestReportsSystem(unittest.TestCase):
 
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The N2 report file, {str(path)} was found but should not have')
+                         f'The N2 report file, {str(path)} was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_basic_problem_reports_argument_n2_only(self):
@@ -408,7 +440,10 @@ class TestReportsSystem(unittest.TestCase):
         self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_basic_problem_reports_argument_n2_and_scaling(self):
@@ -421,6 +456,9 @@ class TestReportsSystem(unittest.TestCase):
         self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertTrue(path.is_file(), f'The scaling report file, {str(path)} was not found')
+        path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+        self.assertFalse(path.is_file(),
+                         f'The optimizer report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_problem_reports_argument_multiple_problems(self):
@@ -429,16 +467,18 @@ class TestReportsSystem(unittest.TestCase):
         # Only problem1 reports should have been generated
 
         # The multiple problem code only runs model so no scaling reports to look for
-        problem_name = 'problem1'
+        problem_name = _default_prob_name()
         problem_reports_dir = pathlib.Path(_reports_dir).joinpath(f'{problem_name}')
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertTrue(path.is_file(), f'The problem1 N2 report file, {str(path)} was not found')
 
-        problem_name = 'problem2'
+        problem_name = problem_name + '2'
         problem_reports_dir = pathlib.Path(_reports_dir).joinpath(f'{problem_name}')
+        self.assertFalse(problem_reports_dir.is_dir(),
+                         f'The problem2 report dir was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The problem2 n2 report file, {str(path)}, was found but should not have')
+                         f'The problem2 n2 report file, {str(path)}, was found but should not exist.')
 
     @hooks_active
     def test_report_generation_basic_problem_reports_dir_argument(self):
@@ -450,10 +490,10 @@ class TestReportsSystem(unittest.TestCase):
         problem_reports_dir = pathlib.Path(custom_reports_dir).joinpath(prob._name)
         path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
         self.assertFalse(path.is_file(),
-                         f'The N2 report file, {str(path)} was found but should not have')
+                         f'The N2 report file, {str(path)} was found but should not exist.')
         path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
         self.assertFalse(path.is_file(),
-                         f'The scaling report file, {str(path)}, was found but should not have')
+                         f'The scaling report file, {str(path)}, was found but should not exist.')
 
 
 @use_tempdirs
@@ -464,9 +504,10 @@ class TestReportsSystemMPI(unittest.TestCase):
     def setUp(self):
         self.n2_filename = _default_n2_filename
         self.scaling_filename = _default_scaling_filename
+        self.optimizer_filename = _default_optimizer_report_filename
 
         # set things to a known initial state for all the test runs
-        openmdao.core.problem._problem_names = []  # need to reset these to simulate separate runs
+        probmod._clear_problem_names()  # need to reset these to simulate separate runs
 
         os.environ.pop('OPENMDAO_REPORTS', None)
         os.environ.pop('OPENMDAO_REPORTS_DIR', None)
@@ -486,29 +527,29 @@ class TestReportsSystemMPI(unittest.TestCase):
             os.environ['TESTFLO_RUNNING'] = self.testflo_running
 
     @hooks_active
-    def test_reports_system_mpi_basic(self):  # example taken from TestScipyOptimizeDriverMPI
+    def test_reports_system_mpi_basic(self):
         prob = om.Problem()
-        prob.model = SellarMDA()
+        prob.model = Diamond()
         prob.driver = om.ScipyOptimizeDriver(optimizer='SLSQP', tol=1e-8)
 
-        prob.model.add_design_var('x', lower=0, upper=10)
-        prob.model.add_design_var('z', lower=0, upper=10)
-        prob.model.add_objective('obj')
-        prob.model.add_constraint('con1', upper=0)
-        prob.model.add_constraint('con2', upper=0)
+        prob.model.add_design_var('iv.x', lower=0, upper=10)
+        prob.model.add_objective('sub.c2.y1')
+        prob.model.add_constraint('sub.c3.y1', upper=0)
 
         prob.setup()
         prob.set_solver_print(level=0)
 
         prob.run_driver()
 
-        # get the path to the problem subdirectory
-        problem_reports_dir = pathlib.Path(_reports_dir).joinpath(prob._name)
+        if prob.comm.rank == 0:
+            problem_reports_dir = pathlib.Path(_reports_dir).joinpath(prob._name)
 
-        path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
-        self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
-        path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
-        self.assertTrue(path.is_file(), f'The scaling report file, {str(path)}, was not found')
+            path = pathlib.Path(problem_reports_dir).joinpath(self.n2_filename)
+            self.assertTrue(path.is_file(), f'The N2 report file, {str(path)} was not found')
+            path = pathlib.Path(problem_reports_dir).joinpath(self.scaling_filename)
+            self.assertTrue(path.is_file(), f'The scaling report file, {str(path)}, was not found')
+            path = pathlib.Path(problem_reports_dir).joinpath(self.optimizer_filename)
+            self.assertTrue(path.is_file(), f'The optimizer report file, {str(path)}, was not found')
 
 
 if __name__ == '__main__':
