@@ -13,7 +13,6 @@ class NodeDisplayData {
         this.filtered = false; // Node is a child to be shown w/partially collapsed parent
         this.filterParent = null; // When filtered, reference to FilterNode container
         this.manuallyExpanded = false; // Node was pre-collapsed but expanded by user
-        this.boxChildren = false; // Surround children with a box in the matrix grid
 
         this.dims = new Dimensions({ x: 1e-6, y: 1e-6, width: 1, height: 1 });
         this.dims.preserve();
@@ -59,10 +58,14 @@ class TreeNode {
     }
 
     _newDisplayData() {
-        if (this.isInputOrOutput()) { this.parent.draw.boxChildren = true; }
         return new NodeDisplayData();
     }
 
+    /** In the matrix grid, draw a box around variables that share the same boxAncestor() */
+    boxAncestor() {
+        return this.isInputOrOutput()? this.parent : null;
+    }
+
     // Accessor functions for this.draw.minimized - whether or not to draw children
     minimize() { this.draw.minimized = true; return this; }
     expand() { this.draw.minimized = false; return this; }
@@ -281,6 +284,8 @@ class TreeNode {
         this.draw.hidden = state.hidden;
         this.filterSelf(state.filtered);
     }
+
+    getTextName() { return this.name; }
 }
 
 /**
@@ -309,6 +314,10 @@ class FilterNode extends TreeNode {
         this.suffix = suffix;
     }
 
+    getTextName() {
+        return `Filtered ${this.suffix[0].toUpperCase() + this.suffix.slice(1)}`;
+    }
+
     /**
      * Add a node to our filtered children, update its state, and make ourselves visible.
      * @param {TreeNode} node Reference to the node to filter.
@@ -423,8 +432,8 @@ class FilterNode extends TreeNode {
  * @property {FilterNode} filter.outputs Children that are outputs to be viewed as collapsed.
  */
 class FilterCapableNode extends TreeNode {
-    constructor(origNode, attribNames) {
-        super(origNode, attribNames);
+    constructor(origNode, attribNames, parent) {
+        super(origNode, attribNames, parent);
     }
 
     // Accessor functions for this.draw.filtered - whether a variable is shown in collapsed form
@@ -448,18 +457,30 @@ class FilterCapableNode extends TreeNode {
         }
     }
 
-    /** Add ourselves to the corrent parental filter */
+    /** Add ourselves to the correct parental filter */
     addSelfToFilter() {
         if (this.isInput()) { this.parent.filter.inputs.add(this); }
         else if (this.isOutput()) { this.parent.filter.outputs.add(this); }
     }
 
-    /** Remove ourselves from the corrent parental filter */
+    /** Remove ourselves from the correct parental filter */
     removeSelfFromFilter() {
         if (this.isInput()) { this.parent.filter.inputs.del(this); }
         else if (this.isOutput()) { this.parent.filter.outputs.del(this); }
     }
 
+    getFilterList() { return [ this.filter.inputs, this.filter.outputs]; }
+
+    wipeFilters() {
+        this.filter.inputs.wipe();
+        this.filter.outputs.wipe();
+    }
+
+    addToFilter(node) {
+        if (node.isInput()) { this.filter.inputs.add(node); }
+        else { this.filter.outputs.add(node); }
+    }
+
     /**
      * Filter ourselves based on the supplied filter state.
      * @param {Boolean} filtered Whether to filter or not.
