@@ -8,9 +8,7 @@ import pathlib
 import numpy as np
 
 from openmdao.core.constants import _SetupStatus, INF_BOUND
-import openmdao.utils.coloring as coloring_mod
 import openmdao.utils.hooks as hooks
-from openmdao.utils.units import convert_units
 from openmdao.utils.mpi import MPI
 from openmdao.utils.webview import webview
 from openmdao.utils.general_utils import ignore_errors, default_noraise
@@ -134,7 +132,7 @@ def _compute_jac_view_info(totals, data, dv_vals, response_vals, coloring):
             # use max of abs value here instead of norm to keep coloring consistent between
             # top level jac and subjacs
             var_matrix[i, j] = np.max(matrix[ofstart:ofend, wrtstart:wrtend])
-            if var_matrix[i, j] > 0. or (coloring and
+            if var_matrix[i, j] > 0. or (coloring is not None and
                                          np.any(mask[ofstart:ofend, wrtstart:wrtend])):
                 nonempty_submats.add((of, wrt))
 
@@ -143,7 +141,7 @@ def _compute_jac_view_info(totals, data, dv_vals, response_vals, coloring):
     for i in range(matrix.shape[0]):
         for j in range(matrix.shape[1]):
             val = matrix[i, j]
-            if coloring and not mask[i, j]:
+            if coloring is not None and not mask[i, j]:
                 val = None
             else:
                 if val == 0.:
@@ -443,21 +441,21 @@ def view_driver_scaling(driver, outfile=_default_scaling_filename, show_browser=
         libs_dir = os.path.join(os.path.dirname(code_dir), 'common', 'libs')
         style_dir = os.path.join(os.path.dirname(code_dir), 'common', 'style')
 
-        with open(os.path.join(code_dir, viewer), "r") as f:
+        with open(os.path.join(code_dir, viewer), "r", encoding='utf-8') as f:
             template = f.read()
 
-        with open(os.path.join(libs_dir, 'tabulator.min.js'), "r") as f:
+        with open(os.path.join(libs_dir, 'tabulator.min.js'), "r", encoding='utf-8') as f:
             tabulator_src = f.read()
 
-        with open(os.path.join(style_dir, 'tabulator.min.css'), "r") as f:
+        with open(os.path.join(style_dir, 'tabulator.min.css'), "r", encoding='utf-8') as f:
             tabulator_style = f.read()
 
-        with open(os.path.join(libs_dir, 'd3.v6.min.js'), "r") as f:
+        with open(os.path.join(libs_dir, 'd3.v6.min.js'), "r", encoding='utf-8') as f:
             d3_src = f.read()
 
         jsontxt = json.dumps(data, default=default_noraise)
 
-        with open(outfile, 'w') as f:
+        with open(outfile, 'w', encoding='utf-8') as f:
             s = template.replace("<tabulator_src>", tabulator_src)
             s = s.replace("<tabulator_style>", tabulator_style)
             s = s.replace("<d3_src>", d3_src)
